#include "Bubble.h"

Bubble::Bubble( QWidget *parent )
        :QWidget( parent, Qt::ToolTip ),
        m_autohideTimer( new QTimer() )
{
    setupUi( this );

    connect( buttonHide, SIGNAL( clicked() ), this, SLOT( hide() ) );
    connect( m_autohideTimer, SIGNAL( timeout() ), this, SLOT( autohide() ) );
}

void Bubble::resize( const QRect &trayIconGeometry )
{
//    int w = layout()->sizeHint().width();

//    if ( w > 250 )
    int w = 250;

    resize( w, heightForWidth( w ) );
    QPixmap ltc( ":images/bubble_left_top_corner.png" );
    QPixmap rtc( ":images/bubble_right_top_corner.png" );
    QPixmap lbc( ":images/bubble_left_bottom_corner.png" );
    QPixmap rbc( ":images/bubble_right_bottom_corner.png" );

    QPixmap maskPixmap( width(), height() );
    maskPixmap.fill( QColor( 0, 0, 0, 0 ) );

    QPainter p( &maskPixmap );

    p.setPen( Qt::black );
    p.setBrush( Qt::black );
    p.drawRect( 0, 10, 10, height() - 20 );
    p.drawRect( width() - 10, 10, 10, height() - 20 );
    p.drawRect( 10, 0, width() - 20, height() );

    p.drawPixmap( 0, 0, ltc );
    p.drawPixmap( 0, height() - 10, lbc );
    p.drawPixmap( width() - 10, 0, rtc );
    p.drawPixmap( width() - 10, height() - 10, rbc );

    this->setMask( maskPixmap.mask() );

    QDesktopWidget desktop;
    QRect desktopGeometry = desktop.availableGeometry( trayIconGeometry.topLeft() );

    if ( trayIconGeometry.x() + width() <= desktopGeometry.width() )
    {
        if ( trayIconGeometry.y() > ( desktopGeometry.height() / 2 ) )
        {
            move( trayIconGeometry.x(), trayIconGeometry.y() - height() );
        }
        else
        {
            move( trayIconGeometry.x(), trayIconGeometry.y() + trayIconGeometry.height() );
        }
    }
    else if ( desktopGeometry.width() - trayIconGeometry.x() + width() > 0 )
    {
        if ( trayIconGeometry.y() > ( desktopGeometry.height() / 2 ) )
        {
            move( desktopGeometry.width() - width(), trayIconGeometry.y() - height() );
        }
        else
        {
            move( desktopGeometry.width() - width(), trayIconGeometry.y() + trayIconGeometry.height() );
        }
    }
    else
    {
        move( desktopGeometry.width() - width(), desktopGeometry.height() - height() );
    }
}

void Bubble::resize( int w, int h )
{
    QWidget::resize( w, h );
}

void Bubble::hide()
{
    m_autohideTimer->stop();

    QWidget::hide();
}

void Bubble::mouseReleaseEvent( QMouseEvent *e )
{
    if ( e->button() == Qt::LeftButton )
        hide();
}

void Bubble::showMessage( const QRect &trayIconGeometry, const QString &title, const QString &msg )
{
    if ( title == titleLabel->text() && msg == messageLabel->text() )
        return;

    titleLabel->setText( title + " " );
    messageLabel->setText( msg );

    resize( trayIconGeometry );
    show();

    m_autohideTimer->start( 10000 );
}

void Bubble::autohide()
{
    if ( !isVisible() )
        return;

    hide();
}
