/*
 * File: MediaManager.h
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#ifndef MEDIAMANAGER_H
#define MEDIAMANAGER_H

#include <QString>
#include <QStringList>
#include <QDebug>
#include <QDBusInterface>
#include <QDBusReply>
#include <QDBusError>
#include <QDir>
#include <QProcess>
#include <QThread>

#include "Volume.h"

/**
  * Manages media that contain packages.
  */
class MediaManager: public QObject
{
    Q_OBJECT
public:
    MediaManager( QThread *parent );
    virtual ~MediaManager();

    int           mountVolume( const QString &udi, const QString &mountPoint );
    int           unmountVolume( const QString &mountPoint, const bool forced /*= false*/ );
    bool          ejectVolume( const QString &udi );
    bool          getVolumeIsMounted( const QString &udi );
    QString       getVolumeMountPoint( const QString &udi );
    QList<Volume> getVolumesList();
    int           getVolumesCount();

private:
    QList<Volume>  m_volumesList;
    QProcess      *m_mountProcess;
    QProcess      *m_unmountProcess;
    QProcess      *m_ejectProcess;

private slots:
    void deviceAdded( const QString &device );
    void deviceRemoved( const QString &device );

signals:
    void volumeAdded( const QString &blockDev, const QString &label, const QString &device );
    void volumeRemoved( const QString &device );
    void volumeAdded();
    void volumeRemoved();
};

#endif
