/*
 * File: ScriptManager.cpp
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#include "ScriptManager.h"


ScriptManager::ScriptManager()
        : m_scriptFile( new QFile() )
{
    autostartScript   = "/etc/init.d/ubudsld";
    pppoaScript       = "/etc/ppp/peers/ubudsl_pppoa";
    pppoeScript       = "/etc/ppp/peers/ubudsl_pppoe";
    papSecretsScript  = "/etc/ppp/pap-secrets";
    chapSecretsScript = "/etc/ppp/chap-secrets";
    optionsScript     = "/etc/ppp/options";

    qDebug() << "autostartScript:"    << autostartScript   << "\npppoaScript"      << pppoaScript
             << "\npppoeScript"       << pppoeScript       << "\npapSecretsScript" << papSecretsScript
             << "\nchapSecretsScript" << chapSecretsScript << "\noptionsScript"    << optionsScript;
}


ScriptManager::~ScriptManager()
{
    qDebug() << "~ScriptManager()";
    delete m_scriptFile;
}


void ScriptManager::setScriptsPermissions()
{
    QFile::setPermissions( optionsScript,     QFile::ReadUser );
    QFile::setPermissions( papSecretsScript,  QFile::ReadUser );
    QFile::setPermissions( chapSecretsScript, QFile::ReadUser );
    QFile::setPermissions( pppoaScript,       QFile::ReadUser|QFile::ReadGroup|QFile::ReadOther );
    QFile::setPermissions( pppoeScript,       QFile::ReadUser|QFile::ReadGroup|QFile::ReadOther );
    QFile::setPermissions( autostartScript,   QFile::ReadUser|QFile::WriteUser|
                                              QFile::ExeUser |QFile::ReadGroup|
                                              QFile::ExeGroup|QFile::ExeOther );

}


bool ScriptManager::writeOptionsScript()
{
    m_scriptFile->setFileName( optionsScript );
    qDebug() << "ScriptManager::writeOptionsScript(): fileName:" << m_scriptFile->fileName();

    if ( m_scriptFile->open( QFile::WriteOnly | QFile::Truncate ) )
    {
#ifndef PCLINUXOS
        m_scriptFile->write( "replacedefaultroute\n" );
#endif
        m_scriptFile->write( "updetach\n" );

        m_scriptFile->close();
        return true;
    }
    else
    {
        return false;
    }
}


bool ScriptManager::writePPPoAScript( const unsigned short &VPI,
                                      const unsigned short &VCI,
                                      const QString &serviceLogin )
{
    m_scriptFile->setFileName( pppoaScript );
    qDebug() << "ScriptManager::writePPPoAScript(): fileName:" << m_scriptFile->fileName();

    if ( m_scriptFile->open( QFile::WriteOnly | QFile::Truncate ) )
    {
        m_scriptFile->write( "lock\n" );
        m_scriptFile->write( "noipdefault\n" );
        m_scriptFile->write( "defaultroute\n" );
        m_scriptFile->write( "hide-password\n" );
        m_scriptFile->write( "lcp-echo-interval 20\n" );
        m_scriptFile->write( "lcp-echo-failure 3\n" );
        m_scriptFile->write( "connect /bin/true\n" );
        m_scriptFile->write( "noauth\n" );
        m_scriptFile->write( "persist\n" );
#ifndef PCLINUXOS
        m_scriptFile->write( "mtu 1492\n" );
#endif
        m_scriptFile->write( "noaccomp\n" );
        m_scriptFile->write( "holdoff 4\n" );
        m_scriptFile->write( "default-asyncmap\n" );
        m_scriptFile->write( QString( "plugin pppoatm.so %1.%2\n" ).arg( VPI )
                             .arg( VCI ).toStdString().c_str() );
        m_scriptFile->write( QString( "user \"%1\"\n" ).arg( serviceLogin )   .toStdString().c_str() );

        m_scriptFile->close();
        return true;
    }
    else
    {
        return false;
    }
}


bool ScriptManager::writePPPoEScript( const QString &serviceLogin )
{
    m_scriptFile->setFileName( pppoeScript );
    qDebug() << "ScriptManager::writePPPoEScript(): fileName:" << m_scriptFile->fileName();

    if ( m_scriptFile->open( QFile::WriteOnly | QFile::Truncate ) )
    {
        m_scriptFile->write( "lock\n" );
        m_scriptFile->write( "noipdefault\n" );
        m_scriptFile->write( "defaultroute\n" );
        m_scriptFile->write( "hide-password\n" );
        m_scriptFile->write( "lcp-echo-interval 20\n" );
        m_scriptFile->write( "lcp-echo-failure 3\n" );
        m_scriptFile->write( "connect /bin/true\n" );
        m_scriptFile->write( "noauth\n" );
        m_scriptFile->write( "persist\n" );
        m_scriptFile->write( "mtu 1492\n" );
        m_scriptFile->write( "noaccomp\n" );
        m_scriptFile->write( "default-asyncmap\n" );
        m_scriptFile->write( "noaccomp\n" );
        m_scriptFile->write( "holdoff 4\n" );
//        m_scriptFile->write( "plugin rp-pppoe.so\n" );
//        m_scriptFile->write ( "nic-nas0\n" );
        m_scriptFile->write( QString( "user \"%1\"\n" ).arg( serviceLogin ).toStdString().c_str() );

        m_scriptFile->close();
        return true;
    }
    else
    {
        return false;
    }
}


bool ScriptManager::writeSecretScripts( const QString &serviceLogin, const QString &servicePassword )
{
    bool ok = true;

    m_scriptFile->setFileName( papSecretsScript );
    qDebug() << "ScriptManager::writeSecretsScript(): fileName:" << m_scriptFile->fileName();

    if ( m_scriptFile->open( QFile::WriteOnly | QFile::Truncate ) )
    {
        m_scriptFile->write( QString( "\"%1\" * \"%2\" *\n" ).arg( serviceLogin ).arg( servicePassword ).toStdString().c_str() );

        m_scriptFile->close();
        ok = true;
    }
    else
    {
        ok = false;
    }


    m_scriptFile->setFileName( chapSecretsScript );
    qDebug() << "ScriptManager::writeSecretsScript(): fileName (2):" << m_scriptFile->fileName();

    if ( m_scriptFile->open( QFile::WriteOnly | QFile::Truncate ) )
    {
        m_scriptFile->write( QString( "\"%1\" * \"%2\" *\n" ).arg( serviceLogin ).arg( servicePassword ).toStdString().c_str() );

        m_scriptFile->close();
        ok = true;
    }
    else
    {
        ok = false;
    }

    if ( ok )
        return true;
    else
        return false;
}


void ScriptManager::replaceInFile( const QString &fileName, const QString &sourceStr, const QString &destStr )
{
    QFile *file = new QFile( fileName );

    if ( file->open( QIODevice::ReadOnly ) )
    {
        //int indexOfBegin;
        //int indexOfEndl;

        QString content = file->readAll();

        file->close();

        while ( content.contains( sourceStr ) )
        {
            /*            indexOfBegin = content.lastIndexOf(sourceStr);
                        indexOfEndl = content.indexOf('\n', indexOfBegin);
                        if ( indexOfBegin >= 0 && indexOfEndl >= 0)
                        {
                            content.remove( indexOfBegin, indexOfEndl );
                        }
            */
            content = content.replace( sourceStr, destStr );
        }

        if ( file->open( QFile::WriteOnly | QFile::Truncate ) )
        {
            QByteArray ba = QByteArray();
            ba.append( content );
            //ba.append( destStr );
            ba.append( "\n" );

            file->write( ba );

            ba.clear();
            file->close();
        }
        else
        {
            qWarning() << "ScriptManager::replaceInFile(): couldn't open file" << fileName << "in write only mode!";
        }
    }
    else
    {
        qWarning() << "ScriptManager::replaceInFile(): couldn't open file" << fileName << "in read only mode!";
    }

    delete file;
}


void ScriptManager::removeFromFile( const QString &fileName, const QString &lineToRemove )
{
    QFile *file = new QFile( fileName );
    if ( file->open( QIODevice::ReadOnly ) )
    {
        QString content = file->readAll();

        while ( content.contains( lineToRemove + "\n" ) )
        {
            content = content.remove( lineToRemove + "\n" );
        }

        file->close();

        if ( file->open( QFile::ReadWrite | QFile::Truncate ) )
        {
            QByteArray ba = QByteArray();
            ba.append( content );
            file->write( ba );
            file->close();
        }
    }

    delete file;
}


void ScriptManager::writeToFile( const QString &fileName, const QString &lineToWrite )
{
    QFile *file = new QFile( fileName );

    if ( file->open( QIODevice::ReadOnly ) )
    {
        QString content = file->readAll();

        if ( content.contains( lineToWrite ) )
        {
            qDebug() << "ScriptManager::writeToFile(): There is already such line.";
            file->close();
            return;
        }

        file->close();

        content = content.append( lineToWrite );
        content = content.append( "\n" );

        if ( file->open( QFile::WriteOnly | QFile::Truncate ) )
        {
            QByteArray ba = QByteArray();
            ba.append( content );
            file->write( ba );
            file->close();
        }
    }
    file->close();
}

void ScriptManager::writeUnicornOption( const QString &optionsFile, int &vpi, int &vci, int &protocol )
{
    QFile options( optionsFile );

    QByteArray toWrite;

    if ( options.open( QFile::ReadOnly ) )
    {
        while ( !options.atEnd() )
        {
            QByteArray line = options.readLine();
            if ( !line.startsWith( "options unicorn_usb_eth" ) )
                toWrite.append( line );
        }

        options.close();
    }

    if ( !toWrite.isEmpty() && !toWrite.endsWith( '\n' ) )
        toWrite.append( '\n' );

    toWrite.append( "options unicorn_usb_eth ActivationMode=3 VPI=" );
    toWrite.append( QString::number( vpi ) );
    toWrite.append( " VCI=" );
    toWrite.append( QString::number( vci ) );
    toWrite.append( "  if_name=zx0" );

    if ( protocol == 0 )
    {
        toWrite.append( " PROTOCOL=pppoatm " );
        toWrite.append( "ENCAPS=vc-encaps\n" );
    }
    else
    {
        toWrite.append( " PROTOCOL=pppoe " );
        toWrite.append( "ENCAPS=llc-encaps\n" );
    }


    if ( options.open( QFile::WriteOnly|QFile::Truncate ) )
    {
        options.write( toWrite );
        options.close();
    }
    else
    {
        qDebug() << "ScriptManager::writeUnicornOption(): failed to write options!";
    }
}

