/*
 * File: SystemInfo.h
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#ifndef SYSTEMINFO_H
#define SYSTEMINFO_H

#include <sys/utsname.h>
#include <QDebug>
#include <QString>
#include <QStringList>
#include <QRegExp>
#include <QFile>

namespace UbuDSL
{
namespace SystemInfo
{
/**
  * @return Version of the currently running system kernel.
  */
QString getKernelRelease();

/**
  * @return Architecture of the running system.
  */
QString getArchitecture();

/**
  * @return Architecture of the processor that system is running on.
  */
QString getProcessorArchitecture();

/**
  * @return Detected Linux distribution's name.
  * If the distribution is not recognised then returns an empty string.
  */
QString getDistributionName();

/**
  * @return Detected Linux distribution's release.
  * If the distribution is not recognised then returns an empty string.
  */
QString getDistributionRelease();
}
}

#endif
