#ifndef CONNECTTHREAD_H_
#define CONNECTTHREAD_H_

#include <QThread>
#include <QByteArray>
#include <QString>
#include <QSettings>
#include <QObject>
#include <QStringList>
#include <QNetworkInterface>
#include <QProcess>
#include <QTimer>
#include <QEventLoop>
#include <QFile>

#include "Globals.h"

#include "libubudsl/Globals.h"
#include "libubudsl/MyProcess.h"
#include "libubudsl/SystemInfo.h"

//extern int getModemState();

class ConnectThread: public QThread
{
    Q_OBJECT
public:
    ConnectThread( QSettings *settings, int ( *fGetModemState )() );
    virtual ~ConnectThread();

    void setModemState( int state );
    void setArguments( const QStringList &arguments ) {
        m_arguments = arguments;
    }
    ReturnState::ReturnState getReturnState();

public slots:
    void run();

private slots:
    bool waitForSynchronisation();
    void pppdProcessError( QProcess::ProcessError error ) { processError( "pppd", error ); }
    void brctlProcessError( QProcess::ProcessError error ) { processError( "br2684ctl", error ); }
    void ifconfigProcessError( QProcess::ProcessError error ) { processError( "ifconfig", error ); }

private:
    int ( *getModemState )();

    void prepare_zx_iface();
    void prepare_nas_iface();
    void prepareBridging();
    void processError( const QString process, QProcess::ProcessError &error );

    void getPPPDArguments( const QString &fileName, QStringList &existingArgs );

    MyProcess *pppd;
    QStringList m_arguments;
    QStringList m_env;
    QSettings  *m_settings;
    int         m_modemState;
    ReturnState::ReturnState m_returnState;

    QString nas_iface;
    QString zx_iface;

signals:
    void debugMsg( QByteArray ba );
    void synchronising();
};

#endif /*CONNECTTHREAD_H_*/
