#include "Globals.h"
#include <QDateTime>
#include <QMutex>
#include <QDir>
#include <QFileInfo>
#include <QFile>
#include <QList>
#include <QNetworkInterface>

bool daemonVerbose = false;

static QMutex mutexLog;
static QFile logFile( UBUDSL_PREFIX + "/share/ubudsl/logs/UbuDSL_Daemon.log" );

bool readUnsignedFromFile( const QString &fileName, unsigned int &value )
{
    FILE* file = fopen( fileName.toLatin1().constData(), "r" );
    if ( file != NULL )
    {
        if ( fscanf( file, "%u", &value ) > 0 )
        {
            fclose( file );
            return true;
        }
        fclose( file );
    }

    return false;
}

bool readLongFromFile( const QString &fileName, unsigned long &value )
{
    FILE* file = fopen( fileName.toLatin1().constData(), "r" );
    if ( file != NULL )
    {
        if ( fscanf( file, "%lu", &value ) > 0 )
        {
            fclose( file );
            return true;
        }
        fclose( file );
    }

    return false;
}

void writeLog( const QString &str )
{
    if ( debug )
        printf( "%s\n", qPrintable( str ) );

    mutexLog.lock();

    if ( logFile.open( QIODevice::WriteOnly|QIODevice::Append ) )
    {
        QByteArray dateArray;
        dateArray.append( QDateTime::currentDateTime().toString( "[dd-MM-yyyy hh:mm:ss] " ) );
        logFile.write( dateArray );
        logFile.write( str.toUtf8() );
        logFile.write( "\n" );
        logFile.close();
    }

    mutexLog.unlock();
}

void createLogPath()
{
    QFileInfo fileInfo( logFile );
    QDir logDir( fileInfo.absolutePath() );

    if ( !logDir.exists() )
        logDir.mkpath( fileInfo.absolutePath() );
}

QString findInterface( const QString baseName, const QString defaultName )
{
    QList<QNetworkInterface> ifaces = QNetworkInterface::allInterfaces();
    QListIterator<QNetworkInterface> ifacesIt( ifaces );

    QRegExp regExp( baseName + "[0-9]{1,2}" );

    while ( ifacesIt.hasNext() )
    {
        QNetworkInterface iface = ifacesIt.next();
        if ( regExp.exactMatch( iface.name() ) )
            return iface.name();
    }

    return defaultName;
}
